@@ -1,6 +1,6 @@
 package Math::Vector::Real;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 use strict;
 use warnings;
@@ -47,6 +47,11 @@ sub zero {
 
 sub is_zero { grep $_, @$_[0] }
 
+sub cube {
+    my ($class, $dim, $size) = @_;
+    bless [($size) x $dim], $class;
+}
+
 sub axis_versor {
     my ($class, $dim, $ix);
     if (ref $_[0]) {
@@ -82,6 +87,12 @@ sub _check_dim {
 
 sub clone { bless [@{$_[0]}] }
 
+sub set {
+    &_check_dim;
+    my ($v0, $v1) = @_;
+    $v0->[$_] = $v1->[$_] for 0..$#$v1;
+}
+
 sub add {
     &_check_dim;
     my ($v0, $v1) = @_;
@@ -195,6 +206,34 @@ sub abs {
     sqrt $acu;
 }
 
+sub abs2 {
+    my $acu = 0;
+    $acu += $_ * $_ for @{$_[0]};
+    $acu;
+}
+
+sub dist {
+    &_check_dim;
+    my ($v0, $v1) = @_;
+    my $d2 = 0;
+    for (0..$#$v0) {
+	my $d = $v0->[$_] - $v1->[$_];
+	$d2 += $d * $d;
+    }
+    sqrt($d2);
+}
+
+sub dist2 {
+    &_check_dim;
+    my ($v0, $v1) = @_;
+    my $d2 = 0;
+    for (0..$#$v0) {
+	my $d = $v0->[$_] - $v1->[$_];
+	$d2 += $d * $d;
+    }
+    $d2;
+}
+
 sub _upgrade {
     my $dim;
     map {
@@ -222,32 +261,118 @@ sub atan2 {
 sub versor {
     my $self = shift;
     my $f = 0;
-    $f += $_ * $_ for @{$_[0]};
+    $f += $_ * $_ for @$self;
     $f == 0 and croak "Illegal division by zero";
     $f = 1/sqrt $f;
     bless [map $f * $_, @$self]
 }
 
 sub wrap {
-    my ($self, $w) = @_;
+    my ($self, $v) = @_;
+    &_check_dim;
     require POSIX;
-    my @r;
-    if (ref $w) {
-        &_check_dim;
-        for (0..$#$self) {
-            my $s1 = $self->[$_];
-            my $w1 = $w->[$_];
-            push @r, $s1 - $w1 * floor($s1/$w1)
+
+    bless [map  { my $s = $self->[$_];
+		  my $c = $v->[$_];
+		  $c - $s * floor($c/$s) } (0..$#$self)];
+}
+
+sub max {
+    my $max = 0;
+    for (@{shift()}) {
+	my $abs = CORE::abs($_);
+	$abs > $max and $max = $abs;
+    }
+    $max
+}
+
+sub min {
+    my $self = shift; 
+    my $min = CORE::abs($self->[0]);
+    for (@$self) {
+	my $abs = CORE::abs($_);
+	$abs < $min and $min = $abs;
+    }
+    $min
+}
+
+sub box {
+    shift;
+    return unless @_;
+    my $min = shift->clone;
+    my $max = $min->clone;
+    my $dim = @$min - 1;
+    for (@_) {
+        for my $ix (0..$dim) {
+            my $c = $_->[$ix];
+            if ($max->[$ix] < $c) {
+                $max->[$ix] = $c;
+            }
+            elsif ($min->[$ix] > $c) {
+                $min->[$ix] = $c
+            }
         }
     }
+    ($min, $max);
+}
+
+sub max_component_index {
+    my $self = shift;
+    return unless @$self;
+    my $max = $self->[0];
+    my $max_ix = 0;
+    for my $ix (1..$#$self) {
+        if ($self->[$ix] > $max) {
+            $max_ix = $ix;
+            $max = $self->[$ix];
+        }
+    }
+    $max_ix;
+}
+
+sub decompose {
+    my ($u, $v) = @_;
+    my $p = $u * ($u * $v)/abs2($u);
+    my $n = $v - $p;
+    wantarray ? ($p, $n) : $n;
+}
+
+sub canonical_base {
+    my ($class, $dim) = @_;
+    my @base = map { bless [(0) x $dim], $class } 1..$dim;
+    $base[$_] = 1 for 0..$#base;
+    return @base;
+}
+
+sub normal_base {
+    my $v = shift;
+    my $dim = @$v;
+    if ($dim == 2) {
+        my $u = $v->versor;
+        @$u = ($u->[1], -$u->[0]);
+        return $u;
+    }
     else {
-        my $iw = 1/$w;
-        for (0..$#$self) {
-            my $s1 = $self->[$_];
-            push @r, $s1 - $w * floor($s1 * $iw);
+        my @base = Math::Vector::Real->canonical_base($dim);
+        $_ = $v->decompose($_) for @base;
+        for my $i (0..$dim - 2) {
+            my $max = abs2($base[$i]);
+            if ($max < 0.3) {
+                for my $j ($i+1..$#base) {
+                    my $d2 = abs2($base[$j]);
+                    if ($d2 > $max) {
+                        @base[$i, $j] = @base[$j, $i];
+                        last unless $d2 < 0.3;
+                        $max = $d2;
+                    }
+                }
+            }
+            my $versor = $base[$i] = $i->versor;
+            $_ = $versor->decompose($_) for @base[$i+1..$#base];
         }
+        pop @base;
+        wantarray ? @base : $base[0];
     }
-    bless \@r;
 }
 
 1;
@@ -339,6 +464,11 @@ Equivalent to C<V(@components)>.
 
 Returns the zero vector of the given dimension.
 
+=item $v = Math::Vector::Real->cube($dim, $size)
+
+Returns a vector of the given dimension with all its components set to
+C<$size>.
+
 =item $u = Math::Vector::Real->axis_versor($dim, $ix)
 
 Returns a unitary vector of the given dimension parallel to the axis
@@ -349,17 +479,68 @@ For instance:
   Math::Vector::Real->axis_versor(5, 3); # V(0, 0, 0, 1, 0)
   Math::Vector::Real->axis_versor(2, 0); # V(1, 0)
 
+=item @b = Math::Vector::Real->canonical_base($dim)
+
+Returns the canonical base for the vector space of the given
+dimension.
+
 =item $u = $v->versor
 
 Returns the versor for the given vector.
 
-=item $wrapped = $v->wrap($w)
+=item $wrapped = $w->wrap($v)
 
-If C<$w> is a vector, returns the result of wrapping the given vector
-in the box defined by C<$w>.
+Returns the result of wrapping the given vector in the box defined by
+C<$w>.
 
-If C<$w> is a scalar, the hypercube of size C<$w> is used as the
-wrapping box.
+=item $max = $v->max
+
+Returns the maximum of the absolute values of the vector components.
+
+=item $min = $v->min
+
+Returns the minimum of the absolute values of the vector components.
+
+=item $d2 = $b->abs2
+
+Returns the norm of the vector squared.
+
+=item $d = $v->dist($u)
+
+Returns the distance between the two vectors.
+
+=item $d = $v->dist2($u)
+
+Returns the distance between the two vectors squared.
+
+=item ($bottom, $top) = Math::Vector::Real->box($v0, $v1, $v2, ...)
+
+Returns the two corners of a hyper-box containing all the given
+vectors.
+
+=item $d = $v->set($u)
+
+Equivalent to C<$v = $u> but without allocating a new object.
+
+Note that this method is destructive.
+
+=item $d = $v->max_component_index
+
+Return the index of the vector component with the maximum size.
+
+=item ($p, $n) = $v->decompose($u)
+
+Decompose the given vector C<$u> in two vectors: one parallel to C<$v>
+and another normal.
+
+In scalar context returns the normal vector.
+
+=item @b = $v->normal_base
+
+Returns a set of vectors forming an ortonormal base for the hyperplane
+normal to $v.
+
+In scalar context returns just some random normal vector.
 
 =back
 
